"""
hudai.resources.article
"""
from ..helpers.resource import Resource


class ArticleResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles')
        self.resource_name = 'Article'

    def list(self,
             article_type=None,
             importance_score_min=None,
             key_term=None,
             link_hash=None,
             person_id=None,
             published_after=None,
             published_before=None,
             page=None):
        return self._list(
            importance_score_min=importance_score_min,
            key_term=key_term,
            link_hash=link_hash,
            person_id=person_id,
            published_after=published_after,
            published_before=published_before,
            type=article_type,
            page=page
        )

    def create(self,
               article_type=None,
               authors=[],
               image_url=None,
               importance_score=None,
               link_url=None,
               published_at=None,
               raw_data_url=None,
               source_url=None,
               text=None,
               title=None):
        return self._create(
            authors=authors,
            image_url=image_url,
            importance_score=importance_score,
            link_url=link_url,
            published_at=published_at,
            raw_data_url=raw_data_url,
            source_url=source_url,
            text=text,
            title=title,
            type=article_type
        )

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id,
               article_type=None,
               authors=[],
               image_url=None,
               importance_score=None,
               link_url=None,
               published_at=None,
               raw_data_url=None,
               source_url=None,
               text=None,
               title=None):
        return self._update(
            entity_id,
            authors=authors,
            image_url=image_url,
            importance_score=importance_score,
            link_url=link_url,
            published_at=published_at,
            raw_data_url=raw_data_url,
            source_url=source_url,
            text=text,
            title=title,
            type=article_type
        )

    def delete(self, entity_id):
        return self._delete(entity_id)

    def key_terms(self, entity_id):
        return self.http_get('/{id}/key-terms',
                             params={'id': entity_id})



"""
hudai.resources.user_contacts
"""
from ..helpers.resource import Resource


class UserContactResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/users/contacts')
        self.resource_name = 'UserContact'

    def list(self, user_id, company_id=None, page=None):
        return self._list(user_id=user_id, company_id=company_id, page=page)

    def create(self,
               user_id,
               company_id,
               name=None,
               email=None,
               phone_number=None):
        return self._create(
            user_id=user_id,
            company_id=company_id,
            name=name,
            email=email,
            phone_number=phone_number)

    def fetch(self, user_id, contact_id):
        return self.http_get('/{id}',
                             query_params={'user_id': user_id},
                             params={'id': contact_id})

    def update(self,
               user_id,
               contact_id,
               company_id=None,
               name=None,
               email=None,
               phone_number=None):
        return self.http_get('/{id}',
                             query_params={'user_id': user_id},
                             params={'id': contact_id},
                             body={
                                 'company_id': company_id,
                                 'name': name,
                                 'email': email,
                                 'phone_number': phone_number,
                             })

    def delete(self, user_id, contact_id):
        return self.http_delete('/{id}',
                                query_params={'user_id': user_id},
                                params={'id': contact_id})



"""
hudai.resources.person
"""
from ..helpers.resource import Resource


class OrganizationResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/organizations')
        self.resource_name = 'Organization'

    def list(self, name=None, email_domain=None, signup_key=None, page=None):
        return self._list(name=name,
                          email_domain=email_domain,
                          signup_key=signup_key,
                          page=page)

    def create(self,
               name=None,
               email_domain=None,
               max_billable_accounts=None,
               signed_license_agreement_at=None):
        return self._create(name=name,
                            email_domain=email_domain,
                            max_billable_accounts=max_billable_accounts,
                            signed_license_agreement_at=signed_license_agreement_at)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id,
               name=None,
               email_domain=None,
               max_billable_accounts=None,
               signed_license_agreement_at=None):
        return self._update(entity_id,
                            name=name,
                            email_domain=email_domain,
                            max_billable_accounts=max_billable_accounts,
                            signed_license_agreement_at=signed_license_agreement_at)

    def delete(self, entity_id):
        return self._delete(entity_id)



"""
hudai.resources.person
"""
from ..helpers.resource import Resource


class PersonResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/people')
        self.resource_name = 'Person'

    def list(self, name=None, title=None, term=None, twitter_handle=None, company_id=None, page=None):
        return self._list(name=name,
                          title=title,
                          term=term,
                          twitter_handle=twitter_handle,
                          company_id=company_id,
                          page=page)

    def create(self, name=None, title=None, twitter_handle=None, linked_in_url=None, image_url=None, source=None, company_id=None):
        return self._create(name=name,
                            title=title,
                            twitter_handle=twitter_handle,
                            linked_in_url=linked_in_url,
                            image_url=image_url,
                            source=source,
                            company_id=company_id)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, entity_id, name=None, title=None, twitter_handle=None, linked_in_url=None, image_url=None, source=None, company_id=None):
        return self._update(entity_id,
                            name=name,
                            title=title,
                            twitter_handle=twitter_handle,
                            linked_in_url=linked_in_url,
                            image_url=image_url,
                            source=source,
                            company_id=company_id)

    def delete(self, entity_id):
        return self._delete(entity_id)

    def quotes(self, entity_id, page=None):
        return self.http_get('/{id}/quotes',
                             params={'id': entity_id, 'page': page})



"""
hudai.resources.article_tag
"""
from ..helpers.resource import Resource


class ArticleTagResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/{article_id}/tags')
        self.resource_name = 'ArticleTag'

    def list(self, article_id, page=None):
        query_params = self._set_limit_offset({'page': page})

        return self.http_get('/', params={'article_id': article_id},
                                  query_params=query_params)

    def create(self, article_id, tag):
        return self.http_post('/', params={'article_id': article_id},
                                   data={'tag': tag})

    def fetch(self, article_id, tag):
        return self.http_get('/{tag}',
                             params={'article_id': article_id, 'tag': tag})

    def delete(self, article_id, tag):
        return self.http_delete('/{tag}',
                                params={'article_id': article_id, 'tag': tag})



"""
hudai.resources.system_event
"""
from ..helpers.resource import Resource


class SystemEventResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/system-events')
        self.resource_name = 'SystemEvent'

    def list(self, page=None):
        return self._list(page=page)

    def create(self, name=None, payload=None):
        return self._create(name=name, payload=payload)

    def fetch(self, entity_id):
        return self._fetch(entity_id)



"""
hudai.resources.article_key_term
"""
from ..helpers.resource import Resource


class ArticleKeyTermResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/{article_id}/key-terms')
        self.resource_name = 'ArticleKeyTerm'

    def list(self, article_id, page=None):
        query_params = self._set_limit_offset({'page': page})

        return self.http_get('/', params={'article_id': article_id},
                                  query_params=query_params)

    def create(self, article_id, term):
        return self.http_post('/', params={'article_id': article_id},
                                   data={'term': term})

    def fetch(self, article_id, term):
        return self.http_get('/{term}',
                             params={'article_id': article_id, 'term': term})

    def delete(self, article_id, term):
        return self.http_delete('/{term}',
                                params={'article_id': article_id, 'term': term})



"""
hudai.helpers.http_client
"""

from datetime import date, datetime
from pydash.chaining import chain
from pydash.objects import map_values
from pydash.strings import camel_case, snake_case
import requests

from .. import __version__

USER_AGENT = 'HUD.ai Python v{version} +({docs})'.format(
    version=__version__,
    docs='https://github.com/FoundryAI/hud-ai-python#readme'
)


class HttpClient(object):
    """
    HTTP client based on `requests` with added convenicenes
        - serializes, deserializes JSON
        - adds auth headers from HudAiClient
    """

    def __init__(self, hud_client, base_url):
        self._hud_client = hud_client
        self._base_url = base_url

    def http_get(self, path, query_params={}, refresh_tokens=True):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.get(self._build_url(path),
                                params=_jsonify(query_params),
                                headers=self._get_headers(refresh_tokens))

        return _pythonify(response.json())

    def http_post(self, path, query_params={}, data={}, refresh_tokens=True):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.post(self._build_url(path),
                                 params=_jsonify(query_params),
                                 json=_jsonify(data),
                                 headers=self._get_headers(refresh_tokens))

        return _pythonify(response.json())

    def http_put(self, path, query_params={}, data={}, refresh_tokens=True):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.put(self._build_url(path),
                                params=_jsonify(query_params),
                                json=_jsonify(data),
                                headers=self._get_headers(refresh_tokens))

        return _pythonify(response.json())

    def http_patch(self, path, query_params={}, data={}, refresh_tokens=True):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.patch(self._build_url(path),
                                  params=_jsonify(query_params),
                                  json=_jsonify(data),
                                  headers=self._get_headers(refresh_tokens))

        return _pythonify(response.json())

    def http_delete(self, path, query_params={}, refresh_tokens=True):
        """
        Wrapped HTTP action that
            - translates Python objects to JSON objects
            - injects the required headers
            - translates the API response back into a Pythonic form
        """
        response = requests.delete(self._build_url(path),
                                   params=_jsonify(query_params),
                                   headers=self._get_headers(refresh_tokens))

        return _pythonify(response.json())

    # Private

    def _build_url(self, path):
        return '{}{}'.format(self._base_url, path)

    def _get_headers(self, refresh_tokens=True):
        headers = {'User-Agent': USER_AGENT}

        if refresh_tokens:
            self._hud_client.refresh_tokens()

        token = self._hud_client.access_token
        if token:
            headers['Authorization'] = 'Bearer {token}'.format(token=token)

        return headers


def _jsonify(value):
    if not isinstance(value, dict):
        return _web_safe(value)

    return chain(value) \
        .map_keys(lambda value, key: camel_case(key)) \
        .map_values(lambda value: _jsonify(value)) \
        .value()

def _pythonify(value):
    if isinstance(value, str):
        try:
            return datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
        except ValueError:
            return value

    if isinstance(value, list):
        return [_pythonify(item) for item in value]

    if isinstance(value, dict):
        return chain(value) \
            .map_keys(lambda value, key: snake_case(key)) \
            .map_values(lambda value: _pythonify(value)) \
            .value()

    return value

def _web_safe(value):
    if isinstance(value, datetime):
        return value.isoformat()

    if isinstance(value, date):
        return value.isoformat()

    if isinstance(value, list):
        return [_web_safe(item) for item in value]

    if isinstance(value, dict):
        return map_values(value, lambda element: _web_safe(element))

    return value



"""
hudai.resources.source
"""
from ..helpers.resource import Resource


class SourceResource(Resource):
    def __init__(self, client):
        Resource.__init__(self, client, base_path='/articles/sources')
        self.resource_name = 'Source'

    def list(self, 
            domain=None,
            name=None, 
            min_reliability=None,
            max_reliability=None,
            created_after=None,
            created_before=None,
            article_id=None,
            page=None):
        return self._list(domain=domain, 
            name=name,
            min_reliability=min_reliability,
            max_reliability=max_reliability,
            created_after=created_after,
            created_before=created_before,
            article_id=article_id,
            page=page)

    def create(self,
            domain,  
            name, 
            reliability_score=None, 
            description=None, 
            language=None, 
            country=None):
        return self._create(domain=domain, 
            name=name, 
            reliability_score=reliability_score,
            description=description,
            language=language,
            country=country)

    def fetch(self, entity_id):
        return self._fetch(entity_id)

    def update(self, 
            entity_id,
            domain=None,  
            name=None, 
            reliability_score=None, 
            description=None, 
            language=None, 
            country=None):
        return self._update(entity_id,
            domain=domain, 
            name=name,  
            reliability_score=reliability_score,
            description=description,
            language=language,
            country=country)

    def delete(self, entity_id):
        return self._delete(entity_id)
        